#!/bin/sh

usage()
{
echo "" >&2
echo "" >&2
echo "Usage: $0 scheme input output" >&2
echo "" >&2
echo "This is a shell script to preprocess scheme files" >&2
echo "in the same way as cpp works for C files." >&2
echo "However the script only recognizes a few 'macros'." >&2
echo "The scipt recognizes 'guile' and 'bigloo' schemes." >&2
echo "" >&2
echo "Below you can find an example scheme script, 'tt.scm'." >&2
echo "The script can be executed by 'guile' or 'bigloo'." >&2
echo "" >&2
echo "To run the script in guile:" >&2
echo "    $ choose.sh guile tt.scm to.scm" >&2
echo "    $ guile -s to.scm" >&2
echo "" >&2
echo "To compile the script in bigloo:" >&2
echo "    $ choose.sh bigloo tt.scm to.scm" >&2
echo "    $ bigloo to.scm" >&2
echo "    $ a.out" >&2
echo "" >&2
echo "--------------- tt.scm -----------------" >&2
echo "; guile-start" >&2
echo "( define-module (sx hello))" >&2
echo "( export hello-func)" >&2
echo "; guile-end" >&2
echo "" >&2
echo "; bigloo-start" >&2
echo "(module hello" >&2
echo "  (export hello-func)" >&2
echo ")" >&2
echo "; bigloo-end" >&2
echo "" >&2
echo "" >&2
echo "(define (hello-func text)" >&2
echo "  (write text)" >&2
echo ")" >&2
echo "" >&2
echo "(hello-func \"Hello World!\")" >&2
echo "--------------- tt.scm -----------------" >&2
echo "" >&2
echo "" >&2
}

if [ $# -ne 3 ]; then
  usage
  exit 1
fi


awk -v scheme=$1 'BEGIN { output = 1; }
/;.*bigloo-start/ { bigloo = 1; line = NR; if(scheme != "bigloo") output = 0; next }
/;.*guile-start/  { guile  = 1; line = NR; if(scheme != "guile")  output = 0; next }
/;.*bigloo-end/   { bigloo = 0; line = NR; if(scheme != "bigloo") output = 1; next }
/;.*guile-end/    { guile  = 0; line = NR; if(scheme != "guile")  output = 1; next }
{
  if(output) print $0
}
END{
if(bigloo)
{
  print "ERROR: 'bigloo' section starting at line " line " was not closed" > "/dev/stderr"
}

if(guile)
  print "ERROR: 'guile' section starting at line " line " was not closed" > "/dev/stderr"
}' $2 > $3

